'''
Various constants extracted from schema.

Created on Oct 15, 2013

@author: pupssman
'''
from enum import Enum


class Status(object):
    FAILED = 'failed'
    BROKEN = 'broken'
    PASSED = 'passed'
    CANCELED = 'canceled'
    PENDING = 'pending'


class Label(object):
    DEFAULT = 'allure_label'
    FEATURE = 'feature'
    STORY = 'story'
    SEVERITY = 'severity'
    ISSUE = 'issue'


class Severity(object):
    BLOCKER = 'blocker'
    CRITICAL = 'critical'
    NORMAL = 'normal'
    MINOR = 'minor'
    TRIVIAL = 'trivial'


class AttachmentType(Enum):

    def __init__(self, mime_type, extension):
        self.mime_type = mime_type
        self.extension = extension

    TEXT = ("text/plain", "txt")
    HTML = ("application/html", "html")
    XML = ("application/xml", "xml")
    PNG = ("image/png", "png")
    JPG = ("image/jpg", "jpg")
    JSON = ("application/json", "json")
    OTHER = ("other", "other")


ALLURE_NAMESPACE = "urn:model.allure.qatools.yandex.ru"
COMMON_NAMESPACE = "urn:model.commons.qatools.yandex.ru"
FAILED_STATUSES = [Status.FAILED, Status.BROKEN]
SKIPPED_STATUSES = [Status.CANCELED, Status.PENDING]



# -*- coding: utf-8 -*-

"""
Created on Feb 23, 2014

@author: pupssman
"""
from six import text_type, iteritems
from contextlib import contextmanager
from functools import wraps
import os
import uuid

from _pytest.runner import Skipped
from _pytest.skipping import XFailed

from lxml import etree
import py

from allure.constants import AttachmentType, Status
from allure.structure import Attach, TestStep, TestCase, TestSuite, Failure, Environment, EnvParameter
from allure.utils import now


class StepContext:

    def __init__(self, allure, title):
        self.allure = allure
        self.title = title
        self.step = None

    def __enter__(self):
        if self.allure:
            self.step = self.allure.start_step(self.title)

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.allure:
            if exc_type is not None:
                if exc_type == Skipped:
                    self.step.status = Status.CANCELED
                elif exc_type == XFailed:
                    self.step.status = Status.PENDING
                else:
                    self.step.status = Status.FAILED
            else:
                self.step.status = Status.PASSED
            self.allure.stop_step()

    def __call__(self, func):
        """
        Pretend that we are a decorator -- wrap the ``func`` with self.
        FIXME: may fail if evil dude will try to reuse ``pytest.allure.step`` instance.
        """
        @wraps(func)
        def impl(*a, **kw):
            with self:
                return func(*a, **kw)
        return impl


class AllureImpl(object):

    """
    Allure test-flow implementation that handles test data creation.
    Devised to be used as base layer for allure adaptation to arbitrary test frameworks.

    Has a SAX-like API -- methods to start or stop a ``TestSuite``, ``TestCase``, ``TestStep`` or to create an ``Attachment``.

    Warning::
      Violation of the call order may result in ill-formed allure XML that wont pass schema validation and render into a report.

    Calls should conform to a rule: A *suite* holds *cases*, a *case* holds *steps*, a *step* holds other *steps*.
    *case* and *step* can have as much *attachments* as they want.

    Therefore, basic call order is as follows::
      allure = AllureImpl('./reports')  # this clears ./reports

      allure.star_suite('demo')
      allure.start_case('test_one')
      allure.stop_case(Status.PASSED)
      allure.start_case('test_two')
      allure.start_step('a demo step')
      allure.attach('some file', 'a quick brown fox..', AttachmentType.TEXT)
      allure.stop_step()
      allure.stop_case(Status.FAILED, 'failed for demo', 'stack trace goes here')
      allure.stop_suite()  # this writes XML into ./reports

    """

    def __init__(self, logdir):
        self.logdir = os.path.normpath(os.path.abspath(os.path.expanduser(os.path.expandvars(logdir))))

        # Delete all files in report directory
        if not os.path.exists(self.logdir):
            os.makedirs(self.logdir)
        else:
            for f in os.listdir(self.logdir):
                f = os.path.join(self.logdir, f)
                if os.path.isfile(f):
                    os.unlink(f)

        # That's the state stack. It can contain TestCases or TestSteps.
        # Attaches and steps go to the object at top of the stack.
        self.stack = []

        self.testsuite = None
        self.environment = {}

    def attach(self, title, contents, attach_type):
        """
        Attaches ``contents`` with ``title`` and ``attach_type`` to the current active thing
        """
        attach = Attach(source=self._save_attach(contents, attach_type=attach_type),
                        title=title,
                        type=attach_type.mime_type)
        self.stack[-1].attachments.append(attach)

    def start_step(self, name):
        """
        Starts an new :py:class:`allure.structure.TestStep` with given ``name``,
        pushes it to the ``self.stack`` and returns the step.
        """
        step = TestStep(name=name,
                        title=name,
                        start=now(),
                        attachments=[],
                        steps=[])
        self.stack[-1].steps.append(step)
        self.stack.append(step)
        return step

    def stop_step(self):
        """
        Stops the step at the top of ``self.stack``
        """
        step = self.stack.pop()
        step.stop = now()

    def start_case(self, name, description=None, labels=None):
        """
        Starts a new :py:class:`allure.structure.TestCase`
        """
        test = TestCase(name=name,
                        description=description,
                        start=now(),
                        attachments=[],
                        labels=labels or [],
                        steps=[])
        self.stack.append(test)

    def stop_case(self, status, message=None, trace=None):
        """
        :arg status: one of :py:class:`allure.constants.Status`
        :arg message: error message from the test
        :arg trace: error trace from the test

        Finalizes with important data the test at the top of ``self.stack`` and returns it

        If either ``message`` or ``trace`` are given adds a ``Failure`` object to the test with them.
        """
        test = self.stack[-1]
        test.status = status
        test.stop = now()

        if message or trace:
            test.failure = Failure(message=message, trace=trace or '')

        self.testsuite.tests.append(test)

        return test

    def start_suite(self, name, description=None, title=None, labels=None):
        """
        Starts a new Suite with given ``name`` and ``description``
        """
        self.testsuite = TestSuite(name=name,
                                   title=title,
                                   description=description,
                                   tests=[],
                                   labels=labels or [],
                                   start=now())

    def stop_suite(self):
        """
        Stops current test suite and writes it to the file in the report directory
        """
        self.testsuite.stop = now()

        with self._reportfile('%s-testsuite.xml' % uuid.uuid4()) as f:
            self._write_xml(f, self.testsuite)

    def store_environment(self):
        if not self.environment:
            return

        environment = Environment(id=uuid.uuid4(), name="Allure environment parameters", parameters=[])
        for key, value in iteritems(self.environment):
            environment.parameters.append(EnvParameter(name=key, key=key, value=value))

        with self._reportfile('environment.xml') as f:
            self._write_xml(f, environment)

    def _save_attach(self, body, attach_type=AttachmentType.TEXT):
        """
        Saves attachment to the report folder and returns file name

        :arg body: str or unicode with contents. str is written as-is in byte stream, unicode is written as utf-8 (what do you expect else?)
        """
        with self._attachfile("%s-attachment.%s" % (uuid.uuid4(), attach_type.extension)) as f:
            if isinstance(body, text_type):
                f.write(body.encode('utf-8'))
            else:
                f.write(body)
            return os.path.basename(f.name)

    @contextmanager
    def _attachfile(self, filename):
        """
        Yields open file object in the report directory with given name
        """
        reportpath = os.path.join(self.logdir, filename)

        with open(reportpath, 'wb') as f:
            yield f

    @contextmanager
    def _reportfile(self, filename):
        """
        Yields open file object in the report directory with given name
        """
        reportpath = os.path.join(self.logdir, filename)
        encoding = 'utf-8'

        logfile = py.std.codecs.open(reportpath, 'w', encoding=encoding)

        try:
            yield logfile
        finally:
            logfile.close()

    def _write_xml(self, logfile, xmlfied):
        logfile.write(etree.tostring(xmlfied.toxml(), pretty_print=True, xml_declaration=False, encoding=text_type))



# encoding: utf-8

'''
This module holds utils for the adaptor

Created on Oct 22, 2013

@author: pupssman
'''

from six import text_type, binary_type, python_2_unicode_compatible, u
from six.moves import filter
import time
import hashlib
import inspect
from traceback import format_exception_only

from _pytest.python import Module

from allure.constants import Label
from allure.structure import TestLabel


def parents_of(item):
    """
    Returns list of parents (i.e. object.parent values) starting from the top one (Session)
    """
    parents = [item]
    current = item

    while current.parent is not None:
        parents.append(current.parent)
        current = current.parent

    return parents[::-1]


def parent_module(item):
    return next(filter(lambda x: isinstance(x, Module), parents_of(item)))


def parent_down_from_module(item):
    parents = parents_of(item)
    return parents[parents.index(parent_module(item)) + 1:]


def sec2ms(sec):
    return int(round(sec * 1000.0))


def uid(name):
    """
    Generates fancy UID uniquely for ``name`` by the means of hash function
    """
    return hashlib.sha256(name).hexdigest()


def now():
    """
    Return current time in the allure-way representation. No further conversion required.
    """
    return sec2ms(time.time())


def labels_of(item):
    """
    Returns list of TestLabel elements.
    """

    def get_marker_that_starts_with(item, name):
        """ get a list of marker object from item node that starts with given
        name or empty list if the node doesn't have a marker that starts with
        that name."""
        suitable_names = filter(lambda x: x.startswith(name), item.keywords.keys())

        markers = list()
        for suitable_name in suitable_names:
            markers.append(item.get_marker(suitable_name))

        return markers

    labels = LabelsList()
    label_markers = get_marker_that_starts_with(item, Label.DEFAULT)
    for label_marker in label_markers:
        label_name = label_marker.name.split('.', 1)[-1]
        for label_value in label_marker.args or ():
            labels.append(TestLabel(name=label_name, value=label_value))

    return labels


def all_of(enum):
    """
    returns list of name-value pairs for ``enum`` from :py:mod:`allure.constants`
    """
    def clear_pairs(pair):
        if pair[0].startswith('_'):
            return False
        if pair[0] in ('name', 'value'):
            return False
        return True
    return filter(clear_pairs, inspect.getmembers(enum))


def unicodify(something):
    if isinstance(something, text_type):
        return something
    elif isinstance(something, binary_type):
        return something.decode('utf-8', errors='replace')
    else:
        try:
            return text_type(something)  # @UndefinedVariable
        except (UnicodeEncodeError, UnicodeDecodeError):
            return u'<nonpresentable %s>' % type(something)  # @UndefinedVariable


def present_exception(e):
    """
    Try our best at presenting the exception in a readable form
    """
    if not isinstance(e, SyntaxError):
        return unicodify('%s: %s' % (type(e).__name__, unicodify(e)))
    else:
        return unicodify(format_exception_only(SyntaxError, e))


def get_exception_message(report):
    """
    get exception message from pytest's internal ``report`` object
    """
    return (getattr(report, 'exception', None) and present_exception(report.exception.value)) or (hasattr(report, 'result') and report.result) or report.outcome


@python_2_unicode_compatible
class LabelsList(list):

    def __eq__(self, other):
        if len(self) != len(other):
            return False

        other = other[:]
        for el in self:
            if el not in other:
                return False

            other.remove(el)

        return True

    def __add__(self, other):
        return self.__class__(super(LabelsList, self).__add__(other))

    def __and__(self, other):
        result = self.__class__()
        for el in self:
            if el in other and el not in result:
                result.append(el)

        return result

    def __str__(self):
        return u(', ').join(map(text_type, [(el.name, el.value) for el in self]))

    def __bytes__(self):
        return self.__str__().encode('utf-8')



# -*- coding: utf-8 -*-

__author__ = 'pupssman'

from six import u, unichr, text_type, binary_type

import re
import sys

from lxml import objectify
from namedlist import namedlist


def element_maker(name, namespace):
    return getattr(objectify.ElementMaker(annotate=False, namespace=namespace,), name)


class Rule(object):
    _check = None

    def value(self, name, what):
        raise NotImplemented()

    def if_(self, check):
        self._check = check
        return self

    def check(self, what):
        if self._check:
            return self._check(what)
        else:
            return True


# see http://en.wikipedia.org/wiki/Valid_characters_in_XML#Non-restricted_characters

# We need to get the subset of the invalid unicode ranges according to
# XML 1.0 which are valid in this python build.  Hence we calculate
# this dynamically instead of hardcoding it.  The spec range of valid
# chars is: Char ::= #x9 | #xA | #xD | [#x20-#xD7FF] | [#xE000-#xFFFD]
# | [#x10000-#x10FFFF]
_legal_chars = (0x09, 0x0A, 0x0d)
_legal_ranges = (
    (0x20, 0x7E),
    (0x80, 0xD7FF),
    (0xE000, 0xFFFD),
    (0x10000, 0x10FFFF),
)
_legal_xml_re = [u("%s-%s") % (unichr(low), unichr(high)) for (low, high) in _legal_ranges if low < sys.maxunicode]
_legal_xml_re = [unichr(x) for x in _legal_chars] + _legal_xml_re
illegal_xml_re = re.compile(u('[^%s]') % u('').join(_legal_xml_re))


def legalize_xml(arg):
    def repl(matchobj):
        i = ord(matchobj.group())
        if i <= 0xFF:
            return u('#x%02X') % i
        else:
            return u('#x%04X') % i
    return illegal_xml_re.sub(repl, arg)


class Element(Rule):

    def __init__(self, name='', namespace=''):
        self.name = name
        self.namespace = namespace

    def value(self, name, what):
        if not isinstance(what, (text_type, binary_type)):
            return self.value(name, str(what))

        if not isinstance(what, text_type):
            try:
                what = text_type(what, 'utf-8')
            except UnicodeDecodeError:
                what = text_type(what, 'utf-8', errors='replace')

        return element_maker(self.name or name, self.namespace)(legalize_xml(what))


class Attribute(Rule):

    def value(self, name, what):
        return str(what)


class Nested(Rule):

    def value(self, name, what):
        return what.toxml()


class Many(Rule):

    def __init__(self, rule, name='', namespace=''):
        self.rule = rule
        self.name = name
        self.namespace = namespace

    def value(self, name, what):
        return [self.rule.value(name, x) for x in what]


class WrappedMany(Many):

    def value(self, name, what):
        values = super(WrappedMany, self).value(name, what)
        return element_maker(self.name or name, self.namespace)(*values)


def xmlfied(el_name, namespace='', fields=[], **kw):
    items = fields + list(kw.items())

    class MyImpl(namedlist('XMLFied', [(item[0], None) for item in items])):

        def toxml(self):
            el = element_maker(el_name, namespace)
            entries = lambda clazz: [(name, rule.value(name, getattr(self, name))) for (name, rule) in items if isinstance(rule, clazz) and rule.check(getattr(self, name))]

            elements = entries(Element)
            attributes = entries(Attribute)
            nested = entries(Nested)
            manys = sum([[(m[0], v) for v in m[1]] for m in entries(Many)], [])

            return el(*([element for (_, element) in elements + nested + manys]),
                      **{name: attr for (name, attr) in attributes})

    return MyImpl



from allure.pytest_plugin import MASTER_HELPER


# providing decorators via allure.xxx instead of pytest.allure.xxx
__methods_to_provide = [
    'step',
    'attach',
    'single_step',
    'label',
    'feature',
    'story',
    'severity',
    'issue',
    'environment',
    'attach_type',
]

for method in __methods_to_provide:
    globals()[method] = getattr(MASTER_HELPER, method)



'''
This holds allure report xml structures

Created on Oct 23, 2013

@author: pupssman
'''

from allure.rules import xmlfied, Attribute, Element, WrappedMany, Nested, Many
from allure.constants import ALLURE_NAMESPACE, COMMON_NAMESPACE


Attach = xmlfied('attachment',
                 source=Attribute(),
                 title=Attribute(),
                 type=Attribute())


Failure = xmlfied('failure',
                  message=Element(),
                  trace=Element('stack-trace'))


TestCase = xmlfied('test-case',
                   name=Element(),
                   title=Element().if_(lambda x: x),
                   description=Element().if_(lambda x: x),
                   failure=Nested().if_(lambda x: x),
                   steps=WrappedMany(Nested()),
                   attachments=WrappedMany(Nested()),
                   labels=WrappedMany(Nested()),
                   status=Attribute(),
                   start=Attribute(),
                   stop=Attribute())


TestSuite = xmlfied('test-suite',
                    namespace=ALLURE_NAMESPACE,
                    name=Element(),
                    title=Element().if_(lambda x: x),
                    description=Element().if_(lambda x: x),
                    tests=WrappedMany(Nested(), name='test-cases'),
                    labels=WrappedMany(Nested()),
                    start=Attribute(),
                    stop=Attribute())


TestStep = xmlfied('step',
                   name=Element(),
                   title=Element().if_(lambda x: x),
                   attachments=WrappedMany(Nested()),
                   steps=WrappedMany(Nested()),
                   start=Attribute(),
                   stop=Attribute(),
                   status=Attribute())

TestLabel = xmlfied('label',
                    name=Attribute(),
                    value=Attribute())


EnvParameter = xmlfied('parameter',
                       name=Element(),
                       key=Element(),
                       value=Element())


Environment = xmlfied('environment',
                      namespace=COMMON_NAMESPACE,
                      id=Element(),
                      name=Element(),
                      parameters=Many(Nested()))



import os
from setuptools import setup


PACKAGE = "pytest-allure-adaptor"
VERSION = "1.5.3"


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


def main():
    setup(
        name=PACKAGE,
        version=VERSION,
        description=("Plugin for py.test to generate allure xml reports"),
        author="pupssman",
        author_email="pupssman@yandex-team.ru",
        packages=["allure"],
        url="https://github.com/allure-framework/allure-python",
        long_description=read('README.rst'),
        entry_points={'pytest11': ['allure_adaptor = allure.pytest_plugin']},
        install_requires=[
            "lxml>=3.2.0",
            "pytest>=2.4.1",
            "namedlist",
            "six",
            "enum34"]
    )

if __name__ == '__main__':
    main()



import argparse
from collections import namedtuple

from _pytest.junitxml import mangle_testnames
import pytest

from allure.common import AllureImpl, StepContext
from allure.utils import LabelsList
from allure.constants import Status, AttachmentType, Severity, \
    FAILED_STATUSES, Label, SKIPPED_STATUSES
from allure.utils import parent_module, parent_down_from_module, labels_of, \
    all_of, get_exception_message
from allure.structure import TestLabel


def pytest_addoption(parser):
    parser.getgroup("reporting").addoption('--alluredir',
                                           action="store",
                                           dest="allurereportdir",
                                           metavar="DIR",
                                           default=None,
                                           help="Generate Allure report in the specified directory (may not exist)")

    severities = [v for (_, v) in all_of(Severity)]

    def severity_label_type(string):
        entries = LabelsList([TestLabel(name=Label.SEVERITY, value=x) for x in string.split(',')])

        for entry in entries:
            if entry.value not in severities:
                raise argparse.ArgumentTypeError('Illegal severity value [%s], only values from [%s] are allowed.' % (entry.value, ', '.join(severities)))

        return entries

    def features_label_type(string):
        return LabelsList([TestLabel(name=Label.FEATURE, value=x) for x in string.split(',')])

    def stories_label_type(string):
        return LabelsList([TestLabel(name=Label.STORY, value=x) for x in string.split(',')])

    parser.getgroup("general").addoption('--allure_severities',
                                         action="store",
                                         dest="allureseverities",
                                         metavar="SEVERITIES_LIST",
                                         default=LabelsList(),
                                         type=severity_label_type,
                                         help="""Comma-separated list of severity names.
                                         Tests only with these severities will be run.
                                         Possible values are:%s.""" % ', '.join(severities))

    parser.getgroup("general").addoption('--allure_features',
                                         action="store",
                                         dest="allurefeatures",
                                         metavar="FEATURES_LIST",
                                         default=LabelsList(),
                                         type=features_label_type,
                                         help="""Comma-separated list of feature names.
                                         Run tests that have at least one of the specified feature labels.""")

    parser.getgroup("general").addoption('--allure_stories',
                                         action="store",
                                         dest="allurestories",
                                         metavar="STORIES_LIST",
                                         default=LabelsList(),
                                         type=stories_label_type,
                                         help="""Comma-separated list of story names.
                                         Run tests that have at least one of the specified story labels.""")


def pytest_configure(config):
    reportdir = config.option.allurereportdir
    if reportdir and not hasattr(config, 'slaveinput'):
        config._allurelistener = AllureTestListener(reportdir, config)
        config.pluginmanager.register(config._allurelistener)
        config.pluginmanager.register(AllureCollectionListener(reportdir))
        pytest.allure._allurelistener = config._allurelistener  # FIXME: maybe we need a different injection mechanism


def pytest_runtest_setup(item):
    item_labels = labels_of(item)

    option = item.config.option
    arg_labels = option.allurefeatures + option.allurestories + option.allureseverities

    if arg_labels and not item_labels & arg_labels:
        pytest.skip('Not suitable with selected labels: %s.' % arg_labels)


class LazyInitStepContext(StepContext):

    """
    This is a step context used for decorated steps.
    It provides a possibility to create step decorators, being initiated before pytest_configure, when no AllureListener initiated yet.
    """

    def __init__(self, allure_helper, title):
        self.allure_helper = allure_helper
        self.title = title
        self.step = None

    @property
    def allure(self):
        return self.allure_helper.get_listener()


class AllureHelper(object):

    """
    This object holds various utility methods used from ``pytest.allure`` namespace, like ``pytest.allure.attach``
    """

    def __init__(self):
        self._allurelistener = None  # FIXME: this gets injected elsewhere, like in the pytest_configure

    def get_listener(self):
        return self._allurelistener

    def attach(self, name, contents, type=AttachmentType.TEXT):  # @ReservedAssignment
        """
        Attaches ``contents`` to a current context with given ``name`` and ``type``.
        """
        if self._allurelistener:
            self._allurelistener.attach(name, contents, type)

    def label(self, name, *value):
        """
        A decorator factory that returns ``pytest.mark`` for a given label.
        """
        allure_label = getattr(pytest.mark, '%s.%s' % (Label.DEFAULT, name))
        return allure_label(*value)

    def severity(self, severity):
        """
        A decorator factory that returns ``pytest.mark`` for a given allure ``level``.
        """
        return self.label(Label.SEVERITY, severity)

    def feature(self, *features):
        """
        A decorator factory that returns ``pytest.mark`` for a given features.
        """
        return self.label(Label.FEATURE, *features)

    def story(self, *stories):
        """
        A decorator factory that returns ``pytest.mark`` for a given stories.
        """

        return self.label(Label.STORY, *stories)

    def issue(self, *issues):
        """
        A decorator factory that returns ``pytest.mark`` for a given issues.
        """
        return self.label(Label.ISSUE, *issues)

    def step(self, title):
        """
        A contextmanager/decorator for steps.

        TODO: when moving to python 3, rework this with ``contextlib.ContextDecorator``.

        Usage examples::

          import pytest

          def test_foo():
             with pytest.allure.step('mystep'):
                 assert False

          @pytest.allure.step('make test data')
          def make_test_data_bar():
              raise ValueError('No data today')

          def test_bar():
              assert make_test_data_bar()

          @pytest.allure.step
          def make_test_data_baz():
              raise ValueError('No data today')

          def test_baz():
              assert make_test_data_baz()

          @pytest.fixture()
          @pytest.allure.step('test fixture')
          def steppy_fixture():
              return 1

          def test_baz(steppy_fixture):
              assert steppy_fixture
        """
        if callable(title):
            return LazyInitStepContext(self, title.__name__)(title)
        else:
            return LazyInitStepContext(self, title)

    def single_step(self, text):
        """
        Writes single line to report.
        """
        if self._allurelistener:
            with self.step(text):
                pass

    def environment(self, **env_dict):
        if self._allurelistener:
            self._allurelistener.impl.environment.update(env_dict)

    @property
    def attach_type(self):
        return AttachmentType

    @property
    def severity_level(self):
        return Severity

    def __getattr__(self, attr):
        """
        Provides fancy shortcuts for severity::

            # these are the same
            pytest.allure.CRITICAL
            pytest.allure.severity(pytest.allure.severity_level.CRITICAL)

        """
        if attr in dir(Severity) and not attr.startswith('_'):
            return self.severity(getattr(Severity, attr))
        else:
            raise AttributeError

MASTER_HELPER = AllureHelper()


def pytest_namespace():
    return {'allure': MASTER_HELPER}


class AllureTestListener(object):

    """
    Listens to pytest hooks to generate reports for common tests.
    """

    def __init__(self, logdir, config):
        self.impl = AllureImpl(logdir)
        self.config = config

        # FIXME: maybe we should write explicit wrappers?
        self.attach = self.impl.attach
        self.start_step = self.impl.start_step
        self.stop_step = self.impl.stop_step

        self.testsuite = None

    def _stop_case(self, report, status=None):
        """
        Finalizes with important data the test at the top of ``self.stack`` and returns it
        """
        [self.attach(name, contents, AttachmentType.TEXT) for (name, contents) in dict(report.sections).items()]

        if status in FAILED_STATUSES:
            self.impl.stop_case(status,
                                message=get_exception_message(report),
                                trace=report.longrepr or report.wasxfail)
        elif status in SKIPPED_STATUSES:
            skip_message = type(report.longrepr) == tuple and \
                report.longrepr[2] or report.wasxfail
            trim_msg_len = 89
            short_message = skip_message.split('\n')[0][:trim_msg_len]

            # FIXME: see pytest.runner.pytest_runtest_makereport
            self.impl.stop_case(status,
                                message=(short_message + '...' *
                                         (len(skip_message) > trim_msg_len)),
                                trace=None if short_message ==
                                skip_message else skip_message)
        else:
            self.impl.stop_case(status)

    def pytest_runtest_protocol(self, __multicall__, item, nextitem):
        if not self.testsuite:
            module = parent_module(item)
            self.impl.start_suite(name='.'.join(mangle_testnames(module.nodeid.split("::"))),
                                  description=module.module.__doc__ or None)
            self.testsuite = 'Yes'

        name = '.'.join(mangle_testnames([x.name for x in parent_down_from_module(item)]))
        self.impl.start_case(name, description=item.function.__doc__, labels=labels_of(item))
        result = __multicall__.execute()

        if not nextitem or parent_module(item) != parent_module(nextitem):
            self.impl.stop_suite()
            self.testsuite = None

        return result

    def pytest_runtest_logreport(self, report):
        if report.passed:
            if report.when == "call":  # ignore setup/teardown
                self._stop_case(report, status=Status.PASSED)
        elif report.failed:
            if report.when != "call":
                self._stop_case(report, status=Status.BROKEN)
            else:
                self._stop_case(report, status=Status.FAILED)
        elif report.skipped:
            if hasattr(report, 'wasxfail'):
                self._stop_case(report, status=Status.PENDING)
            else:
                self._stop_case(report, status=Status.CANCELED)

    def pytest_runtest_makereport(self, item, call, __multicall__):  # @UnusedVariable
        """
        That's the place we inject extra data into the report object from the actual Item.
        """

        report = __multicall__.execute()
        report.__dict__.update(
            exception=call.excinfo,
            result=self.config.hook.pytest_report_teststatus(report=report)[0])  # get the failed/passed/xpassed thingy
        return report

    def pytest_sessionfinish(self):
        if self.testsuite:
            self.impl.stop_suite()
            self.testsuite = None
        self.impl.store_environment()


CollectFail = namedtuple('CollectFail', 'name status message trace')


class AllureCollectionListener(object):

    """
    Listens to pytest collection-related hooks
    to generate reports for modules that failed to collect.
    """

    def __init__(self, logdir):
        self.impl = AllureImpl(logdir)
        self.fails = []

    def pytest_collectreport(self, report):
        if not report.passed:
            if report.failed:
                status = Status.BROKEN
            else:
                status = Status.CANCELED

            self.fails.append(CollectFail(name=mangle_testnames(report.nodeid.split("::"))[-1],
                                          status=status,
                                          message=get_exception_message(report),
                                          trace=report.longrepr))

    def pytest_collection_finish(self):
        """
        Creates a testsuite with collection failures if there were any.
        """

        if self.fails:
            self.impl.start_suite(name='test_collection_phase',
                                  title='Collection phase',
                                  description='This is the tests collection phase. Failures are modules that failed to collect.')
            for fail in self.fails:
                self.impl.start_case(name=fail.name)
                self.impl.stop_case(status=fail.status, message=fail.message, trace=fail.trace)
            self.impl.stop_suite()
